import unittest

from jip.search import SonatypeMavenSearch

class SonatypeMavenSearchTests(unittest.TestCase):
    def setUp(self):
        self.searcher = SonatypeMavenSearch()

    def testGenericSearch(self):
        query = "commons-lang"
        results =self.searcher.search(query)
        self.assertTrue(results is not None)
        self.assertTrue(len(results) > 0)
        self.assertTrue(results[0][0] is not None)
        self.assertTrue(results[0][1] is not None)
        self.assertTrue(results[0][2] is not None)
        self.assertTrue(results[0][3] is not None)

    def testGroupAndArtifactSearch(self):
        group = "commons-lang"
        artif = "commons-lang"
        results =self.searcher.search_group_artifact(group, artif)
        self.assertTrue(results is not None)
        self.assertTrue(len(results) > 0)
        self.assertEqual(results[0][0], group)
        self.assertEqual(results[0][1], group)
        self.assertTrue(results[0][2] is not None)
        self.assertTrue(results[0][3] is not None)

if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env jython
# Copyright (C) 2011 Sun Ning<classicning@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

import os
import sys

import time
try:
    from io import StringIO
except ImportError:
    from StringIO import StringIO
try:
    import queue
except ImportError:
    import Queue as queue
import threading

from jip import __version__, logger

JIP_USER_AGENT = 'jip/%s' % __version__
BUF_SIZE = 4096

class DownloadException(Exception):
    pass


def download(url, target, non_blocking=False, close_target=False, quiet=True, verify=True):
    import requests
    ### download file to target (target is a file-like object)
    if non_blocking:
        pool.submit(url, target, verify)
    else:
        try:
            t0 = time.time()
            source = requests.get(url, verify=verify, headers={'User-Agent': JIP_USER_AGENT})
            source.raise_for_status()
            size = source.headers['Content-Length']
            if not quiet:
                logger.info('[Downloading] %s %s bytes to download' % (url, size))
            for buf in source.iter_content(BUF_SIZE):
                target.write(buf)
            source.close()
            if close_target:
                target.close()
            t1 = time.time()
            if not quiet:
                logger.info('[Downloading] Download %s completed in %f secs' % (url, (t1-t0)))
        except requests.exceptions.RequestException:
            _, e, _ = sys.exc_info()
            raise DownloadException(url, e)


def download_string(url, verify=True):
    import requests
    try:
        response = requests.get(url, verify=verify, headers={'User-Agent': JIP_USER_AGENT})
        response.raise_for_status()
        return response.text
    except requests.exceptions.RequestException:
            _, e, _ = sys.exc_info()
            raise DownloadException(url, e)

class DownloadThreadPool(object):
    def __init__(self, size=3):
        self.queue = queue.Queue()
        self.workers = [threading.Thread(target=self._do_work) for _ in range(size)]
        self.initialized = False

    def init_threads(self):
        for worker in self.workers:
            worker.setDaemon(True)
            worker.start()
        self.initialized = True

    def _do_work(self):
        while True:
            url, target, verify = self.queue.get()
            download(url, target, close_target=True, quiet=False, verify=verify)
            self.queue.task_done()

    def join(self):
        self.queue.join()

    def submit(self, url, target, verify=True):
        if not self.initialized:
            self.init_threads()
        self.queue.put((url, target, verify))

pool = DownloadThreadPool(3)

def get_virtual_home():
    if 'VIRTUAL_ENV' in os.environ:
        JYTHON_HOME = os.environ['VIRTUAL_ENV']
    else:
        ## fail back to use current directory
        JYTHON_HOME = os.getcwd()
    return JYTHON_HOME

def get_lib_path():
    JYTHON_HOME = get_virtual_home()
    DEFAULT_JAVA_LIB_PATH = os.path.join(JYTHON_HOME, 'javalib')

    if not os.path.exists(DEFAULT_JAVA_LIB_PATH):
        os.mkdir(DEFAULT_JAVA_LIB_PATH)
    return DEFAULT_JAVA_LIB_PATH



#! /usr/bin/env jython
# Copyright (C) 2011 Sun Ning<classicning@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

import os
import platform
import sys
import argparse

from jip import logger
from jip.commands import commands


def main():
    if platform.system() == "Windows" and os.getenv("COMSPEC").find("cmd.exe") >= 0:
        os.system("color")
    logger.debug("sys args %s" % sys.argv)
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(dest="command")
    for name, func in commands.items():
        sb = subparsers.add_parser(name, help=func.__doc__)
        for argspec in func.args:
            name, defaults = argspec
            nargs = None if defaults is None else argparse.OPTIONAL
            sb.add_argument(name, nargs=nargs, type=str)

        for option in func.options:
            name, nargs, description,option_type = option
            if nargs == 0:
                sb.add_argument('--'+name, action='store_true',
                        help=description, dest="options."+name)
            else:
                sb.add_argument('--'+name, nargs=nargs, help=description,
                        type=option_type, dest="options."+name)
    
    args = vars(parser.parse_args())
    cmd = args.pop('command')
    options = {}
    for k in list(args.keys()):
        if k.startswith('options.'):
            v = args.pop(k)
            k = k[k.index('.')+1:]
            options[k] = v

    if options:
        args['options'] = options
    logger.debug(args)
    commands[cmd](**args)




#! /usr/bin/env jython
# Copyright (C) 2011 Sun Ning<classicning@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
import codecs

from jip.repository import MavenRepos
from jip.util import get_virtual_home

import os
import shutil

class CacheRepository(MavenRepos):
    def __init__(self):
        self.name = 'cache'
        self.uri = os.path.expanduser(os.path.join(get_virtual_home(),
                                      '.jip', 'cache'))
        if not os.path.exists(self.uri):
            os.makedirs(self.uri)

    def get_artifact_uri(self, artifact, ext):
        directory = self.get_artifact_dir(artifact)
        name = artifact.artifact+"-"+artifact.version+"."+ext

        return os.path.join(self.uri, directory, name)

    def get_artifact_dir(self, artifact):
        directory = os.path.join(self.uri, artifact.group,
                artifact.artifact)
        if not os.path.exists(directory):
            os.makedirs(directory)
        return directory

    def download_jar(self, artifact, local_path=None):
        path = self.get_artifact_uri(artifact, 'jar')
        shutil.copy(path, local_path)

    def download_pom(self, artifact, local_path=None):
        path = self.get_artifact_uri(artifact, 'pom')
        if os.path.exists(path):
            if local_path:
                shutil.copy(path, local_path)

            f = codecs.open(path, mode='r', encoding='utf-8')
            data = f.read()
            f.close()
            return data
        else:
            return None

    def put_pom(self, artifact, data):
        path = self.get_artifact_uri(artifact, 'pom')
        f = codecs.open(path, mode='w', encoding='utf-8')
        f.write(data)
        f.close()

    def put_jar(self, artifact, jarpath):
        path = self.get_artifact_uri(artifact, 'jar')
        shutil.copy(jarpath, path)

class CacheManager(object):
    def __init__(self):
        self.enable = True
        self.cache = CacheRepository()

    def set_enable(self, enable):
        self.enable = enable

    def get_artifact_pom(self, artifact, topath=None):
        if self.enable:
            return self.cache.download_pom(artifact, topath)
        else:
            return None

    def get_artifact_jar(self, artifact, topath):
        self.cache.download_jar(artifact, topath)

    def put_artifact_pom(self, artifact, data):
        self.cache.put_pom(artifact, data)

    def put_artifact_jar(self, artifact, jarpath):
        self.cache.put_jar(artifact, jarpath)

    def as_repos(self):
        return self.cache

    def get_cache_path(self):
        return self.cache.uri

    def get_jar_path(self, artifact, filepath=False):
        if filepath:
            return self.cache.get_artifact_uri(artifact, 'jar')
        else:
            return self.cache.get_artifact_dir(artifact)

    def is_artifact_in_cache(self, artifact, jar=True):
        pom_in_cache = os.path.exists(
                self.cache.get_artifact_uri(artifact, 'pom'))
        jar_in_cache = os.path.exists(
                self.cache.get_artifact_uri(artifact, 'jar'))
        if jar:
            return pom_in_cache and jar_in_cache
        else:
            return pom_in_cache

cache_manager = CacheManager()



#! /usr/bin/env jython
# Copyright (C) 2011 Sun Ning<classicning@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#


import os
import sys
import shutil
import stat
import inspect
import warnings
from string import Template

from jip import repos_manager, index_manager, logger,\
        __version__, __path__, pool, cache_manager
from jip.maven import Pom, Artifact
from jip.util import get_lib_path, get_virtual_home

## command dictionary {name: function}
commands = {}

## options [(name, nargs, description, option_type), ...]
def command(register=True, options=[]):
    def _command(func):
        ## init default repos before running command
        def wrapper(*args, **kwargs):
            repos_manager.init_repos()
            index_manager.initialize()
            func(*args, **kwargs)
            index_manager.finalize()
        ## register in command dictionary
        if register:
            commands[func.__name__.replace('_','-')] = wrapper
            wrapper.__doc__ = inspect.getdoc(func)
            wrapper.__raw__ = func

            ### inspect arguments
            args = inspect.getargspec(func)
            defaults = list(args[3]) if args[3] else []
            wrapper.args = []

            for argidx in range(len(args[0])-1, -1, -1):
                if args[0][argidx] != 'options':
                    default_value = defaults.pop() if defaults else None
                    wrapper.args.append((args[0][argidx], default_value))
                    wrapper.args.reverse()
                else:
                    ## options should have default value {}
                    ## so remove this one for counter
                    defaults.pop()

            ### addtional options
            wrapper.options = options

        return wrapper
    return _command


def _find_pom(artifact, verify=True):
    """ find pom and repos contains pom """
    ## lookup cache first
    if cache_manager.is_artifact_in_cache(artifact):
        pom = cache_manager.get_artifact_pom(artifact)
        return (pom, cache_manager.as_repos())
    else:
        for repos in repos_manager.repos:
            pom = repos.download_pom(artifact, verify)
            ## find the artifact
            if pom is not None:
                cache_manager.put_artifact_pom(artifact, pom)
                return (pom, repos)
        return None


@command(options=[
    ("dry-run", 0, "perform a command without actual download", bool),
    ("copy-pom", 0, "copy pom to library directory", bool),
    ("exclude", "+", "exclude artifacts in install, for instance, 'junit:junit'", str),
    ("insecure", 0, "do not verify the server's TLS certificate", bool)
])
def install(artifact_id, options={}):
    """ Install a package identified by "groupId:artifactId:version" """
    artifact = Artifact.from_id(artifact_id)

    _install([artifact], options=options)


def _resolve_artifacts(artifacts, exclusions=[], verify=True):
    ## download queue
    download_list = []

    ## dependency_set contains artifact objects to resolve
    dependency_set = set()

    for a in artifacts:
        dependency_set.add(a)

    while len(dependency_set) > 0:
        artifact = dependency_set.pop()

        ## to prevent multiple version installed
        ## TODO we need a better strategy to resolve this
        if index_manager.is_same_installed(artifact)\
                and artifact not in download_list:
            continue

        pominfo = _find_pom(artifact, verify)
        if pominfo is None:
            logger.error("[Error] Artifact not found: %s", artifact)
            sys.exit(1)

        if not index_manager.is_installed(artifact):
            pom, repos = pominfo

            # repos.download_jar(artifact, get_lib_path())
            artifact.repos = repos

            # skip excluded artifact
            if not any(map(artifact.is_same_artifact, exclusions)):
                download_list.append(artifact)
                index_manager.add_artifact(artifact)

            pom_obj = Pom(pom)
            for r in pom_obj.get_repositories():
                repos_manager.add_repos(*r)

            more_dependencies = pom_obj.get_dependencies(verify)
            for d in more_dependencies:
                d.exclusions.extend(artifact.exclusions)
                if not index_manager.is_same_installed(d):
                    dependency_set.add(d)

    return download_list


def _install(artifacts, exclusions=[], options={}):
    dryrun = options.get("dry-run", False)
    verify = not options.get("insecure", True)
    if not verify:
        warnings.filterwarnings("ignore", category=Warning)
    _exclusions = options.get('exclude', [])
    copy_pom = options.get('copy-pom', False)
    if _exclusions:
        _exclusions = map(lambda x: Artifact(*(x.split(":"))), _exclusions)
        exclusions.extend(_exclusions)

    download_list = _resolve_artifacts(artifacts, exclusions, verify)

    if not dryrun:
        ## download to cache first
        for artifact in download_list:
            if artifact.repos != cache_manager.as_repos():
                artifact.repos.download_jar(artifact,
                            cache_manager.get_jar_path(artifact), verify)
        pool.join()
        for artifact in download_list:
            cache_manager.get_artifact_jar(artifact, get_lib_path())
            if copy_pom:
                cache_manager.get_artifact_pom(artifact, get_lib_path())

        index_manager.commit()
        logger.info("[Finished] dependencies resolved")
    else:
        logger.info("[Install] Artifacts to install:")
        for artifact in download_list:
            logger.info(artifact)

@command()
def clean():
    """ Remove all downloaded packages """
    logger.info("[Deleting] remove java libs in %s" % get_lib_path())
    shutil.rmtree(get_lib_path())
    index_manager.remove_all()
    index_manager.commit()
    logger.info("[Finished] all downloaded files erased")

## another resolve task, allow jip to resovle dependencies from a pom file.
@command(options=[
    ("dry-run", 0, "perform a command without actual download", bool)
])
def resolve(pomfile, options={}):
    """ Resolve and download dependencies in pom file """
    pomfile = open(pomfile, 'r')
    pomstring = pomfile.read()
    pom = Pom(pomstring)
    ## custom defined repositories
    repositories = pom.get_repositories()
    for repos in repositories:
        repos_manager.add_repos(*repos)

    dependencies = pom.get_dependencies()
    _install(dependencies, options=options)

@command()
def update(artifact_id):
    """ Update a snapshot artifact, check for new version """
    artifact = Artifact.from_id(artifact_id)
    artifact = index_manager.get_artifact(artifact)
    if artifact is None:
        logger.error('[Error] Can not update %s, please install it first' % artifact)
        sys.exit(1)

    if artifact.is_snapshot():
        selected_repos = artifact.repos
        installed_file = os.path.join(get_lib_path(), artifact.to_jip_name())
        if os.path.exists(installed_file):
            lm = os.stat(installed_file)[stat.ST_MTIME]

            ## find the repository contains the new release
            ts = selected_repos.last_modified(artifact)
            if ts is not None and ts > lm :
                ## download new jar
                selected_repos.download_jar(artifact, get_lib_path())

                ## try to update dependencies
                pomstring = selected_repos.download_pom(artifact)
                pom = Pom(pomstring)
                dependencies = pom.get_dependencies()
                _install(dependencies)
        else:
            logger.error('[Error] Artifact not installed: %s' % artifact)
            sys.exit(1)
    else:
        logger.error('[Error] Can not update non-snapshot artifact')
        return

@command()
def version():
    """ Display jip version """
    logger.info('[Version] jip %s, jython %s' % (__version__, sys.version))

@command(options=[
    ("dry-run", 0, "perform a command without actual download", bool)
])
def deps(artifact_id, options={}):
    """ Install dependencies for a given artifact coordinator """
    artifact = Artifact.from_id(artifact_id)
    pominfo = _find_pom(artifact)
    if pominfo is not None:
        pom = Pom(pominfo[0])
        _install(pom.get_dependencies(), options=options)
    else:
        logger.error('[Error] artifact %s not found in any repository' % artifact_id)
        sys.exit(1)

@command(options=[
    ('group', '?', "group name", str),
    ('artifact', '?', "artifact name", str)
])
def search(query="", options={}):
    """ Search maven central repository with keywords"""
    from .search import searcher
    if query is not None and len(query) > 0:
        logger.info('[Searching] "%s" in Maven central repository...' % query)
        results = searcher.search(query)
    else:
        g = options.get('group', '')
        a = options.get('artifact', '')
        logger.info('[Searching] "%s:%s" in Maven central repository...' % (g,a))
        results = searcher.search_group_artifact(g, a)
    if len(results) > 0:
        for item in results:
            g,a,v,p = item
            logger.info("%s-%s (%s)\n\t%s:%s:%s" % (a,v,p,g,a,v))
    else:
        logger.info('[Finished] nothing returned by criteria "%s"' % query)

@command()
def list():
    """ List current installed artifacts """
    index_manager.keep_consistent()
    for a in index_manager.installed:
        logger.info("%s" % a)

@command()
def remove(artifact_id):
    """ Remove an artifact from library path """
    logger.info('[Checking] %s in library index' % artifact_id)
    artifact = Artifact.from_id(artifact_id)
    artifact_path = os.path.join(get_lib_path(), artifact.to_jip_name())

    if index_manager.is_installed(artifact) and os.path.exists(artifact_path):
        os.remove(artifact_path)
        index_manager.remove_artifact(artifact)
        index_manager.commit()
        logger.info('[Finished] %s removed from library path' % artifact_id)
    else:
        logger.error('[Error] %s not installed' % artifact_id)
        sys.exit(1)

@command()
def freeze():
    """ Dump current configuration to a pom file """
    dependencies = index_manager.to_pom()
    repositories = repos_manager.to_pom()
    template = Template(open(os.path.join(__path__[0], '../data/pom.tpl'), 'r').read())
    logger.info( template.substitute({'dependencies': dependencies,
        'repositories': repositories}))



#!/usr/bin/env python
"""Run unittests in this."""

import os
import pkgutil
import sys
import unittest

def suite() :
    loader = unittest.defaultTestLoader
    if len(sys.argv) > 1:
        names = sys.argv[1:]
        test_suite = loader.loadTestsFromNames(names)
    else:
        pkgpath = os.path.dirname(__file__)
        names = [name for _, name,
                 _ in pkgutil.iter_modules([pkgpath])]
        test_suite = loader.loadTestsFromNames(names)
    return test_suite

def main():
    runner = unittest.TextTestRunner()
    result = runner.run(suite())
    if result.wasSuccessful():
        return 0
    else:
        return 1

if __name__ == '__main__':
    sys.exit(main())



#! /usr/bin/env jython
# Copyright (C) 2011 Sun Ning<classicning@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

try:
    import urllib.parse as urlparse
except ImportError:
    import urllib as urlparse
try: import simplejson as json
except ImportError: import json
from jip.util import download_string

class SearchProvider(object):
    def search(self, query, maxresults=20):
        """ Search and return list of tuple(groupId, artifactId, version, packaging)"""
        pass

    def search_group_artifact(self, group, artifact, maxresults=20):
        """ Search with group and artifact name"""
        pass

class SonatypeMavenSearch(SearchProvider):
    query_service = "http://search.maven.org/solrsearch/select?q=%s&rows=%d&wt=json"
    def search(self, query, maxresults=20):
        query_url = self.query_service % (urlparse.quote(query), maxresults)

        data = download_string(query_url)
        
        return self._parse_results(data, version_name='latestVersion') 

    def search_group_artifact(self, group, artifact, maxresults=20):
        query = 'g:"%s" AND a:"%s"'%(group, artifact)
        query_url = self.query_service % (urlparse.quote(query), maxresults)
        query_url += "&core=gav"

        data = download_string(query_url)
        return self._parse_results(data, version_name='v')

    def _parse_results(self, data, version_name):
        results = json.loads(data)
       
        docs = results['response']['docs']
        return [(doc['g'], doc['a'], doc[version_name], doc['p']) for doc in docs]

searcher = SonatypeMavenSearch()




#! /usr/bin/env jython
# Copyright (C) 2011 Sun Ning<classicning@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#


import os
import sys
import re
from xml.etree import ElementTree
from string import Template, whitespace

from . import logger, repos_manager, cache_manager

class Artifact(object):
    def __init__(self, group, artifact, version=None):
        self.group = group
        self.artifact = artifact
        self.version = version
        self.timestamp = None
        self.build_number = None
        self.exclusions = []
        self.repos = None

    def to_jip_name(self, pattern="$artifact-$version.$ext", ext="jar"):
        template = Template(pattern)
        filename = template.substitute({'group':self.group, 'artifact':self.artifact,
                'version': self.version, 'ext': ext})
        return filename

    def to_maven_name(self, ext):
        group = self.group.replace('.', '/')
        return "%s/%s/%s/%s-%s.%s" % (group, self.artifact, self.version, self.artifact, self.version, ext)

    def to_maven_snapshot_name(self, ext):
        group = self.group.replace('.', '/')
        version_wo_snapshot = self.version.replace('-SNAPSHOT', '')
        return "%s/%s/%s/%s-%s-%s-%s.%s" % (group, self.artifact, self.version, self.artifact, version_wo_snapshot,
                self.timestamp, self.build_number, ext)

    def __eq__(self, other):
        if isinstance(other, Artifact):
            return other.group == self.group and other.artifact == self.artifact and other.version == self.version
        else:
            return False

    def __str__(self):
        return "%s:%s:%s" % (self.group, self.artifact, self.version)

    def __repr__(self):
        return self.__str__()

    def __hash__(self):
        ## python3 set requires a custom hash function to ensure item hash unchanged
        ## however, python2 pickle module might call this function before internal
        ## propertoes loaded. So here we return 0 if no key property loaded as a workaround
        if hasattr(self, 'group') and hasattr(self, 'artifact') and hasattr(self, 'version'):
            return self.group.__hash__()*13+self.artifact.__hash__()*7+self.version.__hash__()
        else:
            return 0

    def is_snapshot(self):
        return self.version.find('SNAPSHOT') > 0

    def is_same_artifact(self, other):
       ## need to support wildcard
       group_match = True if self.group == '*' or other.group == '*' else self.group == other.group
       artif_match = True if self.artifact == '*' or other.artifact == '*' else self.artifact == other.artifact
       return group_match and artif_match

    @classmethod
    def from_id(cls, artifact_id):
        group, artifact, version = artifact_id.split(":")
        artifact = Artifact(group, artifact, version)
        return artifact


class WhitespaceNormalizer(ElementTree.TreeBuilder, object):   # The target object of the parser
     def data(self, data):
         data=data.strip(whitespace)         #strip whitespace at start and end of string
         return super(WhitespaceNormalizer,self).data(data)

class Pom(object):
    def __init__(self, pom_string):
        self.pom_string = pom_string
        self.eletree = None
        self.properties = None
        self.dep_mgmt = None
        self.parent = None

    def get_element_tree(self):
        if self.eletree is None:
            ## we use this dirty method to remove namesapce attribute so that elementtree will use default empty namespace
            pom_string = re.sub(r"<project(.|\s)*?>", '<project>', self.pom_string, 1)
            parser = ElementTree.XMLParser(target=WhitespaceNormalizer())
            parser.feed(pom_string.encode('utf-8'))
            self.eletree = parser.close()
        return self.eletree

    def get_parent_pom(self, verify=True):
        if self.parent is not None:
            return self.parent

        eletree = self.get_element_tree()
        parent = eletree.find("parent")
        if parent is not None:
            parent_group_id = parent.findtext("groupId")
            parent_artifact_id = parent.findtext("artifactId")
            parent_version_id = parent.findtext("version")

            artifact = Artifact(parent_group_id, parent_artifact_id, parent_version_id)
            if cache_manager.is_artifact_in_cache(artifact, jar=False):
                parent_pom = cache_manager.get_artifact_pom(artifact)
            else:
                for repos in repos_manager.repos:
                    parent_pom = repos.download_pom(artifact, verify)
                    if parent_pom is not None:
                        cache_manager.put_artifact_pom(artifact, parent_pom)
                        break

            if parent_pom is not None:
                self.parent = Pom(parent_pom)
                return self.parent
            else:
                logger.error("cannot find parent pom %s" % parent_pom)
                sys.exit(1)
        else:
            return None

    def get_dependency_management(self, verify=True):
        if self.dep_mgmt is not None:
            return self.dep_mgmt

        dependency_management_version_dict = {}

        parent = self.get_parent_pom(verify)
        if parent is not None:
            dependency_management_version_dict.update(parent.get_dependency_management())

        properties = self.get_properties()
        eletree = self.get_element_tree()
        dependency_management_dependencies = eletree.findall("dependencyManagement/dependencies/dependency")

        for dependency in dependency_management_dependencies:
            group_id = self.__resolve_placeholder(dependency.findtext("groupId"), properties)
            artifact_id = self.__resolve_placeholder(dependency.findtext("artifactId"), properties)
            version = self.__resolve_placeholder(dependency.findtext("version"), properties)

            version_val = version
            while True:
                try:
                    version = version_val if version_val != None else version
                    version_val = properties.get(version_val[2:-1])
                except TypeError:
                    break


            scope = dependency.findtext("scope")
            if scope is not None and scope == 'import':
                artifact = Artifact(group_id, artifact_id, version)
                global repos_manager
                for repos in repos_manager.repos:
                    import_pom = repos.download_pom(artifact)
                    if import_pom is not None:
                        break
                if import_pom is not None:
                    import_pom = Pom(import_pom)
                    dependency_management_version_dict.update(import_pom.get_dependency_management())
                else:
                    logger.error("[Error] can not find dependency management import: %s" % artifact)
                    sys.exit(1)
            else:
                ## will also remember scope for scope inheritance
                dependency_management_version_dict[(group_id, artifact_id)] = (version, scope)

        self.dep_mgmt = dependency_management_version_dict
        return dependency_management_version_dict

    def get_dependencies(self, verify=True):
        dep_mgmt = self.get_dependency_management(verify)
        props = self.get_properties()
        eletree = self.get_element_tree()

        runtime_dependencies = []

        dependencies = eletree.findall("dependencies/dependency")
        for dependency in dependencies:
            # resolve placeholders in pom (properties and pom references)
            group_id = self.__resolve_placeholder(dependency.findtext("groupId"), props)
            artifact_id = self.__resolve_placeholder(dependency.findtext("artifactId"), props)
            version = dependency.findtext("version")
            if version is not None:
                version = self.__resolve_placeholder(version, props)

            scope = dependency.findtext("scope")
            optional = dependency.findtext("optional")

            ### dependency exclusion
            ### there is no `version` in a exclusion definition
            exclusions = []
            for exclusion in dependency.findall("exclusions/exclusion"):
                groupId = exclusion.findtext("groupId")
                artifactId = exclusion.findtext("artifactId")
                excluded_artifact = Artifact(groupId, artifactId, None)
                exclusions.append(excluded_artifact)

            # runtime dependency
            if optional is None or optional == 'false':
                if version is None:
                    version = dep_mgmt[(group_id, artifact_id)][0]
                if scope is None:
                    ## bug fix, scope is an optional attribute
                    if (group_id, artifact_id) in dep_mgmt:
                        scope = dep_mgmt[(group_id, artifact_id)][1]
                if scope in (None, 'runtime', 'compile'):
                    artifact = Artifact(group_id, artifact_id, version)
                    artifact.exclusions = exclusions
                    runtime_dependencies.append(artifact)

        logger.debug('Find dependencies: %s'% runtime_dependencies)
        return runtime_dependencies

    def get_properties(self):
        if self.properties is not None:
            return self.properties

        eletree = self.get_element_tree()
        # parsing in-pom properties
        properties = {}
        properties_ele = eletree.find("properties")
        if properties_ele is not None:
            prop_eles = list(properties_ele)
            for prop_ele in prop_eles:
                if prop_ele.tag == 'property':
                    name = prop_ele.get("name")
                    value = prop_ele.get("value")
                else:
                    name = prop_ele.tag
                    value = prop_ele.text
                properties[name] = value

        parent = self.get_parent_pom()
        if parent is not None:
            properties.update(parent.get_properties())

        ## pom specific elements
        groupId = eletree.findtext('groupId')
        artifactId = eletree.findtext('artifactId')
        version = eletree.findtext('version')
        if version is None:
            version = eletree.findtext('parent/version')
        if groupId is None:
            groupId = eletree.findtext('parent/groupId')

        properties["project.parent.version"] = eletree.findtext('parent/version')
        properties["project.parent.groupId"] = eletree.findtext('parent/groupId')
        properties["project.groupId"] = self.__resolve_placeholder(groupId, properties)
        properties["project.artifactId"] = artifactId
        properties["project.version"] = self.__resolve_placeholder(version, properties)

        properties["pom.groupId"] = groupId
        properties["pom.artifactId"] = artifactId
        properties["pom.version"] = version
        self.properties = properties
        return properties

    def __resolve_placeholder(self, text, properties):
       def subfunc(matchobj):
            key = matchobj.group(1)
            if key in properties:
                return properties[key]
            else:
                return matchobj.group(0)
       return re.sub(r'\$\{(.*?)\}', subfunc, text)

    def get_repositories(self):
        eletree = self.get_element_tree()

        repositories = eletree.findall("repositories/repository")
        repos = []
        for repository in repositories:
            name = repository.findtext("id")
            uri = repository.findtext("url")
            repos.append((name, uri, "remote"))
        return repos



#! /usr/bin/env jython
# Copyright (C) 2011 Sun Ning<classicning@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser
import codecs
from string import Template
import os
import locale
import shutil
import stat
import sys
import time
import hashlib
try:
    import urllib.error as urlerror
    import urllib.request as urlrequest
except ImportError:
    import urllib2 as urlerror
    import urllib2 as urlrequest
from xml.etree import ElementTree

from jip import logger
from jip.util import DownloadException, download, download_string, get_virtual_home

class RepositoryManager(object):
    MAVEN_LOCAL_REPOS = ('local', os.path.expanduser(os.path.join('~', '.m2', 'repository')), 'local')
    MAVEN_PUBLIC_REPOS = ('public', "https://repo1.maven.org/maven2/", 'remote')
    def __init__(self):
        self.repos = []

    def add_repos(self, name, uri, repos_type, order=None):
        if repos_type == 'local':
            repo = MavenFileSystemRepos(name, uri)
        elif repos_type == 'remote':
            repo = MavenHttpRemoteRepos(name, uri)
        else:
            logger.warn('[Error] Unknown repository type.')
            sys.exit(1)

        if repo not in self.repos:
            if order is not None:
                self.repos.insert(order, repo)
            else:
                self.repos.append(repo)
            logger.debug('[Repository] Added: %s' % repo.name)

    def _load_config(self):
        config_file_path = os.path.join(get_virtual_home(), '.jip_config')
        if not os.path.exists(config_file_path):
            config_file_path = os.path.expanduser(os.path.join('~', '.jip_config'))
        if os.path.exists(config_file_path):
            config = ConfigParser()
            config.read(config_file_path)

            repos = []
            ## only loop section starts with "repos:"
            repos_sections = filter(lambda x:x.startswith("repos:"), config.sections())
            for section in repos_sections:
                name = section.split(':')[1]
                uri = config.get(section, "uri")
                rtype = config.get(section, "type")
                repos.append((name, uri, rtype))
            return repos
        else:
            return None

    def init_repos(self):
        for repo in (self._load_config() or [self.MAVEN_PUBLIC_REPOS]):
            ## create repos in order
            name, uri, rtype = repo
            self.add_repos(name, uri, rtype, order=len(self.repos))

    def to_pom(self):
        pom_template = Template("""
        <repository>
            <id>$repoId</id>
            <name>$repoId</name>
            <url>$url</url>
        </repository>""")
        reps = []
        for repo in self.repos:
            ### remote only
            if isinstance(repo, MavenHttpRemoteRepos):
                ### remote repository other than default
                if repo.uri != self.MAVEN_PUBLIC_REPOS[1]:
                    content = pom_template.substitute({'repoId':repo.name,
                        'url': repo.uri})
                    reps.append(content)
        return ''.join(reps)


## globals
repos_manager = RepositoryManager()

class MavenRepos(object):
    def __init__(self, name, uri):
        self.name = name
        self.uri = uri

    def __eq__(self, other):
        if isinstance(other, MavenRepos):
            return self.uri == other.uri
        else:
            return False

    def get_artifact_uri(self, artifact, ext):
        pass

    def download_jar(self, artifact, local_path):
        """ download or copy file to local path, raise exception when failed """
        pass

    def download_pom(self, artifact, verify=True):
        """ return a content string """
        pass

    def last_modified(self, artifact):
        """ return last modified timestamp """
        pass

    def download_check_sum(self, checksum_type, origin_file_name):
        """ return pre calculated checksum value, only avaiable for remote repos """
        pass

class MavenFileSystemRepos(MavenRepos):
    def __init__(self, name, uri):
        MavenRepos.__init__(self, name, os.path.expanduser(uri))

    def get_artifact_uri(self, artifact, ext):
        maven_name = artifact.to_maven_name(ext)
        maven_file_path = os.path.join(self.uri,maven_name)
        return maven_file_path

    def download_jar(self, artifact, local_path):
        maven_file_path = self.get_artifact_uri(artifact, 'jar')
        logger.info("[Checking] jar package from %s" % self.name)
        if os.path.exists(maven_file_path):
            local_jip_path = os.path.join(local_path, artifact.to_jip_name())
            logger.info("[Downloading] %s" % maven_file_path)
            shutil.copy(maven_file_path, local_jip_path)
            logger.info("[Finished] %s completed" % local_jip_path)
        else:
            logger.error("[Error] File not found %s" % maven_file_path)
            raise IOError('File not found:' + maven_file_path)

    def download_pom(self, artifact, verify=True):
        maven_file_path = self.get_artifact_uri(artifact, 'pom')
        logger.info('[Checking] pom file %s'% maven_file_path)
        if os.path.exists(maven_file_path):
            pom_file = codecs.open(maven_file_path, mode='r', encoding='utf-8')
            data = pom_file.read()
            pom_file.close()
            return data
        else:
            logger.info('[Skipped] pom file not found at %s' % maven_file_path)
            return None

    def last_modified(self, artifact):
        maven_file_path = self.get_artifact_uri(artifact, 'pom')
        if os.path.exists(maven_file_path):
            last_modify = os.stat(maven_file_path)[stat.ST_MTIME]
            return last_modify
        else:
            return None

class MavenHttpRemoteRepos(MavenRepos):
    def __init__(self, name, uri):
        MavenRepos.__init__(self, name, uri)
        self.pom_cache = {}
        self.pom_not_found_cache = []

    def download_jar(self, artifact, local_path, verify=True):
        maven_path = self.get_artifact_uri(artifact, 'jar')
        logger.info('[Downloading] jar from %s' % maven_path)
        local_jip_path = os.path.join(local_path, artifact.to_jip_name())
        local_f = open(local_jip_path, 'wb')
        ## download jar asyncly
        download(maven_path, local_f, True, verify=verify)
        ##logger.info('[Finished] %s downloaded ' % maven_path)

    def download_pom(self, artifact, verify=True):
        if artifact in self.pom_not_found_cache:
            return None

        if artifact in self.pom_cache:
            return self.pom_cache[artifact]

        if artifact.is_snapshot():
            snapshot_info = self.get_snapshot_info(artifact)
            if snapshot_info is not None:
                ts, bn = snapshot_info
                artifact.timestamp = ts
                artifact.build_number = bn

        maven_path = self.get_artifact_uri(artifact, 'pom')
        try:
            logger.info('[Checking] pom file %s'% maven_path)
            data = download_string(maven_path, verify)
            ## cache
            self.pom_cache[artifact] = data

            return data
        except DownloadException:
            self.pom_not_found_cache.append(artifact)
            logger.info('[Skipped] Pom file not found at %s'% maven_path)
            return None

    def get_artifact_uri(self, artifact, ext):
        if not artifact.is_snapshot():
            maven_name = artifact.to_maven_name(ext)
        else:
            maven_name = artifact.to_maven_snapshot_name(ext)
        if self.uri.endswith('/'):
            maven_path = self.uri + maven_name
        else:
            maven_path = self.uri + '/' + maven_name
        return maven_path

    def get_snapshot_info(self, artifact):
        metadata_path = self.get_metadata_path(artifact)

        try:
            data = download_string(metadata_path)

            eletree = ElementTree.fromstring(data)
            timestamp = eletree.findtext('versioning/snapshot/timestamp')
            build_number = eletree.findtext('versioning/snapshot/buildNumber')

            return (timestamp, build_number)
        except DownloadException:
            return None

    def get_metadata_path(self, artifact):
        group = artifact.group.replace('.', '/')
        metadata_path = "%s/%s/%s/%s/maven-metadata.xml" % (self.uri, group,
                artifact.artifact, artifact.version)
        return metadata_path

    def last_modified(self, artifact):
        metadata_path = self.get_metadata_path(artifact)
        try:
            fd = urlrequest.urlopen(metadata_path)
            if 'last-modified' in fd.headers:
                ts = fd.headers['last-modified']
                fd.close()
                locale.setlocale(locale.LC_TIME, 'en_US')
                last_modified = time.strptime(ts, '%a, %d %b %Y %H:%M:%S %Z')
                return time.mktime(last_modified)
            else:
                fd.close()
                return 0
        except urlerror.HTTPError:
            return None

    def download_check_sum(self, checksum_type, origin_file_name):
        """ return pre calculated checksum value, only avaiable for remote repos """
        checksum_url = origin_file_name + "." + checksum_type
        try:
            return download_string(checksum_url)
        except DownloadException:
            return None

    def checksum(self, filepath, checksum_type):
        if checksum_type == 'md5':
            hasher = hashlib.md5()
        elif checksum_type == 'sha1':
            hasher = hashlib.sha1()

        buf_size = 1024*8
        file_to_check = open(filepath, 'r')
        buf = file_to_check.read(buf_size)
        while len(buf) > 0:
            hasher.update(buf)
            buf = file_to_check.read(buf_size)

        file_to_check.close()
        return hasher.hexdigest()



import unittest

from jip.repository import MavenFileSystemRepos
from jip.maven import Artifact
from contextlib import contextmanager
import shutil
import tempfile

import errno    
import os

@contextmanager
def tmpdir():
    dirname = tempfile.mkdtemp()
    try:
        yield dirname
    finally:
        shutil.rmtree(dirname)

def mkdir_p(path):
    try:
        os.makedirs(path)
    except OSError as exc:
        if exc.errno == errno.EEXIST and os.path.isdir(path):
            pass
        else:
            raise

class RespositoryTests(unittest.TestCase):

    def testLocalDownloadOfPomWithUtfChars(self):
        with tmpdir() as temp_repo_dir:
            artifact_dir = "%s/dummygroup/dummyartifact/1.0.0/" % temp_repo_dir
            mkdir_p(artifact_dir)
            with open("%sdummyartifact-1.0.0.pom" % artifact_dir, "w") as f:
                f.write("\xe2")
            testee = MavenFileSystemRepos('dummy_name', temp_repo_dir)
            artifact = Artifact('dummygroup', 'dummyartifact', '1.0.0')
            pom = testee.download_pom(artifact)
            # should not raise an UnicodeDecodeError
            pom.encode("utf-8")

if __name__ == '__main__':
    unittest.main()



import unittest

from jip import repos_manager, index_manager
from jip.maven import Artifact
from jip.commands import _resolve_artifacts

class InstallTests(unittest.TestCase):
    def setUp(self):
        repos_manager.init_repos()
        index_manager.initialize()

    def testArtifact(self):
        commons_lang = Artifact('commons-lang', 'commons-lang', '2.6')
        self.assertEqual(str(commons_lang), 'commons-lang:commons-lang:2.6')

    def testResolve(self):
        junit = Artifact('junit', 'junit', '3.8.1')
        artifacts = _resolve_artifacts([junit])
        self.assertEqual(len(artifacts), 1)

    def testResolveArtifactWithUmlautsInPom(self):
        artifact = Artifact('de.l3s.boilerpipe', 'boilerpipe', '1.1.0')
        artifacts = _resolve_artifacts([artifact])
        self.assertEqual(len(artifacts), 1)

    def testExclusion(self):
        pig = Artifact('org.apache.pig', 'pig', '0.8.3')
        exclusion = [Artifact(*x.split(":")) for x in ['ant:ant', 'junit:junit','org.eclipse.jdt:core']]
        artifacts = _resolve_artifacts([pig], exclusion)
        self.assertTrue(len(artifacts) > 0)
        self.assertFalse(any([x.is_same_artifact(Artifact('ant', 'ant')) for x in artifacts]))
        self.assertFalse(any([x.is_same_artifact(Artifact('junit', 'junit')) for x in artifacts]))
        self.assertFalse(any([x.is_same_artifact(Artifact('org.eclipse', 'core')) for x in artifacts]))


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env jython
# Copyright (C) 2011 Sun Ning<classicning@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

import os
import sys
import pickle
import threading
from string import Template

from jip import logger
from jip.util import get_virtual_home, get_lib_path

class IndexManager(object):
    """An IndexManager persists the artifacts you installed in your path and 
    keep it consistent"""
    def __init__(self, filepath):
        self.filepath  = filepath
        self.installed = set()
        self.committed = False
        self.persist_lock = threading.Lock()

    def add_artifact(self, artifact):
        self.installed.add(artifact)
        #self.persist()
        
    def get_artifact(self, artifact_eq):
        for artifact in self.installed:
            if artifact == artifact_eq:
                return artifact
        return None

    def remove_artifact(self, artifact):
        a = self.get_artifact(artifact)
        if a is not None:
            self.installed.remove(a)

    def remove_all(self):
        self.installed = set()

    def is_installed(self, artifact_test):
        return self.get_artifact(artifact_test) is not None

    def is_same_installed(self, artifact):
        is_same = lambda a: a.is_same_artifact(artifact)
        return any(map(is_same, self.installed))

    def persist(self):
        if not self.committed:
            return 
        try:
            self.persist_lock.acquire()

            picklefile = open(self.filepath, 'wb')
            pickle.dump(self.installed, picklefile)
            picklefile.close()
        finally:
            self.persist_lock.release()

    def initialize(self):
        self.committed = False
        if os.path.exists(self.filepath):
#            try:
#                pickledata = open(self.filepath, 'rb').read()
#                artifacts = pickle.loads(pickledata)
#                for artifact in artifacts: self.installed.add(artifact)
#                self.keep_consistent()
#            except:
#                pass
             pickledata = open(self.filepath, 'rb').read()
             artifacts = pickle.loads(pickledata)
             for artifact in artifacts: self.installed.add(artifact)
             self.keep_consistent()

    def keep_consistent(self):
        in_path_libs = os.listdir(get_lib_path())
        remembered_libs = list(self.installed)
        for artifact in remembered_libs:
            local_name = artifact.to_jip_name()
            if local_name not in in_path_libs:
                logger.warn(('[Warning] %s is not in your path which should be installed by previous actions. '
                    + 'If you still need it, please reinstall it by: jip install %s') % (local_name, str(artifact)))
                self.remove_artifact(artifact)

    def finalize(self):
        self.persist()

    def commit(self):
        self.committed = True

    def to_pom(self):
        pom_template = Template("""
        <dependency>
            <groupId>$groupId</groupId>
            <artifactId>$artifactId</artifactId>
            <version>$version</version>
        </dependency>""")
        deps = []
        for artifact in self.installed:
            content = pom_template.substitute({'groupId': artifact.group,
                'artifactId': artifact.artifact, 'version': artifact.version}) 
            deps.append(content)

        return ''.join(deps)

index_manager = IndexManager(os.path.join(get_virtual_home(), '.jip_index'))




#! /usr/bin/env jython
# Copyright (C) 2011 Sun Ning<classicning@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from jip import logger, repos_manager
from jip.commands import resolve as jip_resolve, command
from jip.commands import _install as jip_install
from jip.maven import Artifact
from jip.util import pool

try:
    from setuptools import setup as _setup
    from setuptools.command.install import install as _install
except:    
    from distutils.core import setup as _setup
    from distutils.command.install import install as _install
from distutils.core import Command as _Command    

dist_pom = 'pom.xml'
dependencies = []
repositories = []
exclusions = []
use_pom = True

def requires_java(requires_info):
    global use_pom, dependencies, repositories, exclusions
    use_pom = False
    if 'repositories' in requires_info:
        repositories = requires_info['repositories']
    if 'dependencies' in requires_info:
        dependencies = [Artifact(*d) for d in requires_info['dependencies']]
    if 'exclusions' in requires_info:
        exclusions = [Artifact(*d) for d in requires_info['exclusions']]

@command(register=False)
def requires_java_install():
    for repos in repositories:
        repos_manager.add_repos(repos[0], repos[1], 'remote')
    jip_install(dependencies, exclusions)
    pool.join()
    logger.info("[Finished] all dependencies resolved")

class install(_install):
    def run(self):
        _install.run(self)
        logger.info('running jip_resolve')
        self.run_command('resolve')

class resolve(_Command):
    user_options = []

    def initialize_options(self):
        self.pom_file = dist_pom

    def finalize_options(self):
        pass

    def run(self):
        if use_pom:
            jip_resolve(self.pom_file)
        else:
            requires_java_install()

def setup(**kwargs):
    if 'requires_java' in kwargs:
        requires_java(kwargs.pop('requires_java'))

    if 'pom' in kwargs:
        dist_pom = kwargs.pop('pom')

    if 'cmdclass' not in kwargs:
        kwargs['cmdclass'] = {}

    if 'install' not in kwargs['cmdclass']:
        ## add install command if user doesn't define custom one
        kwargs['cmdclass']['install'] = install
    if 'resolve' not in kwargs['cmdclass']:
        ## add resolve command        
        kwargs['cmdclass']['resolve'] = resolve

    return _setup(**kwargs)




import sys
if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

from jip.embed import require

def is_jython():
    return sys.platform.lower().startswith('java')

class JipRequireTests(unittest.TestCase):
    @unittest.skipUnless(is_jython(), "requires jython")
    def testRequire(self):
        require("commons-lang:commons-lang:2.6")
        from org.apache.commons.lang import StringUtils

        self.assertEquals(StringUtils.removeStart("jip rocks", "jip"), " rocks")
        self.assertEquals(StringUtils.reverse("jip rocks"), "skcor pij")

if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2011-2015 Sun Ning<classicning@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

long_description = open('README.rst').read()

def is_virtualenv():
    import os
    return 'VIRTUAL_ENV' in os.environ

setup_args=dict(
        name="jip",
        version="0.9.16",
        author="Sun Ning",
        author_email="classicning@gmail.com",
        url="https://github.com/sunng87/jip",
        description="jip installs packages, for Jython and Python",
        license='mit',
        packages=['jip'],
        long_description=long_description,
        classifiers=['Development Status :: 4 - Beta',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',
            'Topic :: Software Development :: Build Tools',
            'Programming Language :: Python :: 2.6',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Java',
            'Environment :: Console',
            'Operating System :: POSIX']
)

setup_args['scripts'] = ["scripts/jython-all"]
requires = ['requests']
if sys.version_info < (2, 7):
    requires.append('argparse')
setup_args['install_requires'] = requires
setup_args['entry_points'] = {
            'console_scripts' : [
                'jip = jip.main:main'
            ]
        }
setup_args['data_files'] = [('data', ['data/pom.tpl'])]

setup(**setup_args)




#! /usr/bin/env jython
# Copyright (C) 2011 Sun Ning<classicning@gmail.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

import sys

from jip import cache_manager, repos_manager, pool
from jip.commands import _resolve_artifacts
from jip.maven import Artifact

__all__ = ['require']

def _prepare():
    repos_manager.init_repos()

def require(artifact_id):
    _prepare()

    artifact = Artifact.from_id(artifact_id)
    artifact_set = _resolve_artifacts([artifact])

    ## download to cache first
    for artifact in artifact_set:
        if artifact.repos != cache_manager.as_repos():
             artifact.repos.download_jar(artifact, 
                 cache_manager.get_jar_path(artifact))
    pool.join()

    ## append jars to path
    for artifact in artifact_set:
        sys.path.append(cache_manager.get_jar_path(artifact, filepath=True))
                 

